<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Category extends Ecart_Db_Table 
{
    protected $_name = 'catalog_category';
    protected $_primary = 'id';
    
    /**
     * Inactive category ids. Including
     * active child categories of inactive parents
     *  
     * @var array
     */
    private $_disabledCategories = null;

    /**
     *
     * @param array $data
     * @param int $parentId
     * @return int
     */
    public function insertItem(array $data, $parentId = 0)
    {
        $db = $this->getAdapter();
        $parentRow = $this->fetchRow(array($db->quoteInto('id = ?', $parentId)));
        
        if (!$parentRow)  {
            return false;
        }
        $siteId = $parentRow->site_id;
        
        $this->update(
            array('lft' => new Zend_Db_Expr('lft + 2')),
            array(
                'site_id = ' . $siteId,
                'lft > ' . $parentRow->rgt,
            )
        );
        $this->update(
            array('rgt' => new Zend_Db_Expr('rgt + 2')),
            array(
                'site_id = ' . $siteId,
                'rgt >= ' . $parentRow->rgt
            )
        );
        
        $data['site_id'] = $siteId;
        $data['lvl'] = $parentRow->lvl + 1;
        $data['lft'] = $parentRow->rgt;
        $data['rgt'] = $parentRow->rgt + 1;
        return $this->insert($data);
    }

    /**
     *
     * @param int $id
     * @return <type>
     */
    public function deleteItem($id)
    {
        $db = $this->getAdapter();
        $row = $this->fetchRow($db->quoteInto('id = ?', $id));
        /*
         * select cid array
         * delete from this where id in cid array
         * delete from description where cat_id in cid array
         * update tree index
         */
        
        $idForDelete = $db->fetchCol("SELECT id FROM {$this->_name} 
        WHERE lft BETWEEN $row->lft AND $row->rgt AND site_id = $row->site_id");
        
        if (!sizeof($idForDelete)) return false;
        
        $db->delete(self::$_db_prefix . 'catalog_category_description', $db->quoteInto('category_id IN(?)', $idForDelete));
        $db->delete(self::$_db_prefix . 'catalog_product_category', $db->quoteInto('category_id IN(?)', $idForDelete));
        Ecart::single('catalog/hurl')->delete(array(
            $db->quoteInto('key_id IN(?)', $idForDelete),
            "key_type = 'c'"
        ));
        
        $nstreeTable = new Ecart_NSTree_Table();
        return $nstreeTable->deleteNode($id, true);
    }
    
    /**
     * Retrieve the set of parent categories
     * 
     * @param int $categoryId
     * @param int $languageId [optional]
     * @return array Keys are category ids
     */
    public function getParentItems($categoryId, $languageId = null)
    {
        if (null === $languageId) {
            $languageId = Ecart::getLanguageId();
        }
        $languageId = intval($languageId);
        $select = $this->getAdapter()->select();
        $select->from(array('c' => $this->_name), array())
            ->join(array('c2' => $this->_name), 'c.lft BETWEEN c2.lft AND c2.rgt', array('id', 'lvl'))
            ->join(array('hu' => self::$_db_prefix . 'catalog_hurl'), 'hu.key_id = c2.id', array('key_word'))
            ->joinLeft(array('cd' => self::$_db_prefix . 'catalog_category_description'), 'cd.category_id = c2.id AND cd.language_id = ' . $languageId,
                array('name', 'meta_title', 'meta_description', 'meta_keyword'))
            ->where($this->getAdapter()->quoteInto('c.id = ?', $categoryId))
            ->where('c2.site_id = c.site_id')
            ->where('hu.key_type = "c"')
            ->order('c2.lft');

        return $this->getAdapter()->fetchAssoc($select);
    }

    /**
     *
     * @param int $categoryId
     * @param bool $deep
     * @param bool $activeOnly
     * @return <type>
     */
    public function getChildItems($categoryId, $deep = false, $activeOnly = false)
    {
        $db = $this->getAdapter();
        $select = $db->select();
        
        if (!$categoryId) { //get root items
            $select->from(array('c' => $this->_name))
                ->joinLeft(array('cd' => self::$_db_prefix . 'catalog_category_description'), 
                'cd.category_id = c.id AND cd.language_id = ' . Ecart_Locale::getLanguageId(), 
                'name')
                ->where("c.lvl = 0")
                ->order('c.id ASC');
            
            return $db->fetchAll($select->__toString());
        } else { //get child items
            $row = $this->find($categoryId)->current();
            $select->from(array('c' => $this->_name))
                ->joinLeft(array('cd' => self::$_db_prefix . 'catalog_category_description'), 
                    'cd.category_id = c.id AND cd.language_id = ' . Ecart_Locale::getLanguageId(), 
                    'name')
                ->joinLeft(array('ch' => self::$_db_prefix . 'catalog_hurl'),
                    "c.id = ch.key_id AND ch.key_type = 'c'",
                    'key_word')
                ->where("c.lft BETWEEN {$row->lft} AND {$row->rgt}")
                ->where("c.site_id = {$row->site_id}")
                ->order('c.lft');
                
                if (!$deep) {
                    $select->where("c.lvl = " . ($row->lvl + 1));
                }
                
                if ($activeOnly) {
                    $select->where("c.cat_status = 'enabled'");
                }
                    
            return $db->fetchAll($select->__toString());
        }
    }

    /**
     *
     * @param int $languageId
     * @param mixed(array) $siteIds
     * @param bool $isActive
     * @return array
     */
    public function getFlatTree($languageId, $siteIds = null, $activeOnly = false)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('c' => $this->_name), array('id', 'site_id', 'lvl', 'lft', 'rgt'))
            ->joinLeft(
                array('cd' => self::$_db_prefix . 'catalog_category_description'), 
                'cd.category_id = c.id AND cd.language_id = ' . $languageId,
                array('cd.name'));
        
        if ($siteIds) {
            $where = $this->getAdapter()->quoteInto('c.site_id IN(?)', $siteIds);
            $select->where($where);
        }
        
        if ($activeOnly && $disabledCategories = $this->getDisabledIds()) {
            $select->where('c.id NOT IN (?)', $disabledCategories);
        }
        
        $select->joinLeft(array('hu' => self::$_db_prefix . 'catalog_hurl'), 'hu.key_id = c.id', array('hu.key_word') );
        $select->where("hu.key_type='c'");
        $select->order('c.lft');
        $result = $select->query();
        $tree = array();
        
        while (($row = $result->fetch())) {
            $tree[$row['site_id']][] = $row;
        }
        
        return $tree;
    }
    
    /**
     *
     * @param int $siteId
     * @return array
     */
    public function getSiteCategories($siteId) 
    {
        $where = $this->getAdapter()->quoteInto('site_id = ? ', $siteId);
        return $this->getAdapter()->fetchCol("SELECT id FROM {$this->_name} WHERE $where");
    }
    
    /**
     *
     * @param <type> $categoryId
     * @return mixed
     */
    public function getInfoWithKeyWord($categoryId)
    {
        if (!$categoryId) {
            return false;
        }
             
        return $this->getAdapter()->fetchRow("SELECT c.*, hu.key_word FROM {$this->_name} c " .
            "LEFT JOIN " . self::$_db_prefix . "catalog_hurl hu ON hu.key_id = c.id AND hu.key_type = 'c' " .
            "WHERE c.id = :id", 
            array('id' => $categoryId)
        );
    }
    
    /**
     * Retrieve the list or products
     *
     * @param Ecart_Db_Row|'uncategorized' $category
     * @param integer $limit
     * @param integer $start
     * @param string $order
     * @return array
     */
    public function getListProducts($category, $limit, $start, $order)
    {
        if ($category == 'uncategorized') { //querying for uncategorized products
            $select = "SELECT p.id, 
                pd.name, 
                p.is_active, 
                pf.product_id as is_featured, 
                pf.date_exp, 
                p.price, 
                p.date_available
            FROM " . self::$_db_prefix . "catalog_product AS p
            LEFT JOIN " . self::$_db_prefix . "catalog_product_description AS pd 
                ON pd.product_id = p.id AND pd.language_id = " . Ecart::getLanguageId() . "
            LEFT JOIN " . self::$_db_prefix . "catalog_product_featured AS pf ON pf.product_id = p.id
            WHERE p.id <> ALL (
                SELECT product_id FROM " . self::$_db_prefix . "catalog_product_category AS ptc
            )
            OR p.id = ANY (
                SELECT product_id
                FROM " . self::$_db_prefix . "catalog_product_category AS ptc
                INNER JOIN {$this->_name} AS c ON c.id = ptc.category_id
                WHERE c.lvl = 0
            )
            ORDER BY {$order}
            LIMIT {$start}, {$limit}";
            
            $result = $this->getAdapter()->fetchAll($select);
            return $result;
        }
        
        if (!is_object($category)) {
            return false;
        }
        
        $select = $this->getAdapter()->select();
        
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'), 
            array(new Zend_Db_Expr('DISTINCT(p.`id`)'), 'is_active', 'price', 'date_available')
        );
        $select->joinLeft(
            array('ptc' => self::$_db_prefix . 'catalog_product_category'),
            'p.id = ptc.product_id',
            array()
        );
        if ($category->lvl == 0) {
            $select->joinLeft(array('c' => $this->_name),
                          'c.id = ptc.category_id',
                          array())
                   ->where('c.site_id = ?', $category->site_id);
        } else {
            $select->where('ptc.category_id = ?', $category->id);
        }
        $select->joinLeft(
                array('pd' => self::$_db_prefix . 'catalog_product_description'), 
                "pd.product_id = p.id AND pd.language_id = " . Ecart::getLanguageId(),
                array('name')
            )
            ->joinLeft(
                array('pf' => self::$_db_prefix . 'catalog_product_featured'),
                "pf.product_id = p.id",
                array('product_id', 'date_exp')
            )
            ->limit($limit, $start)
            ->order($order);
        $dataset = array();
        $result = $this->getAdapter()->query($select);
        while (($row = $result->fetch())) {
            $dataset[] = array(
                'id' => $row['id'], 
                'name' => $row['name'], 
                'is_active' => $row['is_active'],
                'is_featured' => isset($row['product_id']) ? 1 : 0,
                'date_exp' => $row['date_exp'],
                'price' => $row['price'],
                'date_available' => $row['date_available']
            );
        }
        return $dataset;
    }
    
    /**
     *
     * @param string $url
     * @param int $siteId
     * @return <type>
     */
    public function getIdByUrl($url, $siteId)
    {
        $query = "SELECT c.id 
            FROM {$this->_name} AS c
            INNER JOIN " . self::$_db_prefix . "catalog_hurl as hu ON c.id = hu.key_id
            WHERE hu.key_word = '$url'
                AND hu.key_type = 'c'
                AND hu.site_id = $siteId";
                
        return $this->getAdapter()->fetchOne($query);
    }
    
    /**
     * Retrieve the array of sites, where requested categories are lied in.
     * 
     * @param array $catIds
     * @return array|null
     */
    public function getSiteByCategory($catIds = null)
    {
        if (!is_array($catIds)) {
            return null;
        }
        
        $query = "SELECT c.site_id 
            FROM {$this->_name} AS c
            WHERE c.id IN (?)";
        
        return $this->getAdapter()->fetchCol($query, implode(',', $catIds));
    }
    
    /**
     * Retrieve root category of site
     * 
     * @param int $siteId
     * @return Ecart_Db_Table_Row
     */
    public function getRoot($siteId = null)
    {
        if (null === $siteId) {
            $siteId = Ecart::getSiteId();
        }
        $select = $this->select()
            ->where("site_id = {$siteId}")
            ->where("lvl = 0");
            
        return $this->fetchRow($select);
    }
    
    /**
     * Retrieve disabled category ids including 
     * enabled childs of disabled category
     * 
     * @return array
     */
    public function getDisabledIds()
    {
        if (null === $this->_disabledCategories) {
            $categories = $this->fetchAll("cat_status = 'enabled' AND site_id = " . Ecart::getSiteId(), 'lft');
            $disabledCategories = $this->fetchAll("cat_status = 'disabled' AND site_id = " . Ecart::getSiteId(), 'lft');
            
            $result = array();
            foreach ($disabledCategories as $disabledCategory) {
                $result[$disabledCategory['id']] = $disabledCategory['id'];
                foreach ($categories as $category) {
                    if ($category['lft'] > $disabledCategory['lft'] && $category['rgt'] < $disabledCategory['rgt']) {
                        $result[$category['id']] = $category['id'];
                    }
                }
            }
            $this->_disabledCategories = array_values($result);
        }
        return $this->_disabledCategories;
    }
}