<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Catalog
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Catalog_Model_Product extends Ecart_Db_Table 
{
    protected $_name = 'catalog_product';
    protected $_rowClass = 'Ecart_Catalog_Model_Product_Row';
    protected $_dependentTables = array(
        'Ecart_Catalog_Model_Product_Description',
        'Ecart_Catalog_Model_Product_Featured',
        'Ecart_Catalog_Model_Product_Variation', 
        'Ecart_Catalog_Model_Product_Attribute',
        'Ecart_Catalog_Model_Product_Stock',
        'Ecart_Catalog_Model_Product_Gallery'
    );
    
    /**
     * Update or insert product.
     * Transmitting product quantity and weight is required
     *  when updateing existing product.
     * Returns last saved product
     * 
     * @param array $data
     * @param int $siteId
     * @return Ecart_Catalog_Model_Product_Row
     */
    public function save($data)
    {
        foreach ($data as $id => $values) {
            if (!$id || !$row = $this->find($id)->current()) {
                unset($id);
                $row = $this->createRow();
            } else {
                $oldQuantity = $row->quantity;
                $row->quantity = $values['quantity'];
                Ecart::dispatch('catalog_product_update_quantity', array(
                    'product' => $row,
                    'stock' => Ecart::single('catalog/product_stock')->find($id)->current(),
                    'old_quantity' => $oldQuantity
                ));
            }
            if (empty($values['weight'])) {
                $values['weight'] = 0;
            }
            $row->setFromArray($values);
            $row->is_active = $values['is_active'];
            $row->save();
        }
        Ecart::message()->addSuccess(
            Ecart::translate('core')->__(
                'Data was saved successfully'
            )
        );
        return $row;
    }
    
    /**
     * Only Sitemap is use it
     * Retrieve the list of all available products
     * 
     * @param int $languageId
     * @param array $siteIds
     * @return array
     */
    public function getAllActiveProducts($languageId, $siteIds = false) 
    {
        $today = Ecart_Date::now()->toPhpFormat()->toString('Y-m-d');
        $select = $this->getAdapter()->select();
        $select->from(array('ptc' => self::$_db_prefix . 'catalog_product_category'), array())
            ->joinLeft(
                array('p' => self::$_db_prefix . 'catalog_product'),
                'p.id = ptc.product_id', 
                array('id')
            )
            ->joinLeft(
                array('pd' => self::$_db_prefix . 'catalog_product_description'), 
                "pd.product_id = p.id AND pd.language_id = {$languageId} ",
                array('name')
            )
            ->joinLeft(
                array('hu' => self::$_db_prefix . 'catalog_hurl'), 
                "hu.key_id = p.id AND hu.key_type='p'", 
                array('key_word'))
            ->where('p.is_active = 1')
            ->where('p.date_available IS NULL OR p.date_available <= ?', $today);
        if ($siteIds) {
            $select->joinLeft(
                array('c' => self::$_db_prefix . 'catalog_category'), 
                'ptc.category_id = c.id', 
                array()
            );
            $where = $this->getAdapter()->quoteInto('c.site_id IN(?)', $siteIds);
            $select->where($where);
        }
        $select->distinct();
        return $select->query()->fetchAll();
    }
    
    public function getProductInfo($productId, $languageId)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('p' => self::$_db_prefix . 'catalog_product'))
            ->joinLeft(
                array('pd' => self::$_db_prefix . 'catalog_product_description'),
                'pd.product_id = p.id',
                array('name', 'description')
            )
            ->where('p.id = ?', $productId)
            ->where('pd.language_id = ?', $languageId);
        return $this->getAdapter()->fetchRow($select->__toString());
    }
    
    public function getProductCategories($productId, $languageId)
    {
        $select = $this->getAdapter()->select();
        $select->from(
                   array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                   array('product_id','category_id')
                )
               ->joinLeft(
                   array('cd' => self::$_db_prefix . 'catalog_category_description'),
                   'cd.category_id = ptc.category_id', 
                   array('name as category_name')
               )
              ->where('ptc.product_id = ?', $productId)
              ->where('cd.language_id = ?', $languageId);
        return $select->query()->fetchAll();
    }
    
    public function getCount()
    {
        return $this->count();
    }
    
    /**
     * Retrieve count of products inside category
     * 
     * @param int $categoryId
     * @param string $mode(normal|recursive)
     * @return int
     */
    public function countByCategoryId($categoryId, $mode = 'normal')
    {
        if ($mode == 'recursive') {
            if (!$category = Ecart::single('catalog/category')->find($categoryId)->current()) {
                return false;
            }
            $select = $this->getAdapter()->select();
            $select->from(
                array('p' => self::$_db_prefix . 'catalog_product'),
                array(new Zend_Db_Expr('COUNT(p.id)'))
            );
            $select->join(
                array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                'p.id = ptc.product_id',
                array())
                ->join(array('c' => self::$_db_prefix . 'catalog_category'),
                'c.id = ptc.category_id',
                array())
                ->where('c.site_id = ?', $category->site_id)
                ->where("c.lft BETWEEN $category->lft AND $category->rgt")
                ->where("c.rgt BETWEEN $category->lft AND $category->rgt")
                ->group('p.id');
                
            return $this->getAdapter()->fetchOne($select);
        } else {
            return $this->getAdapter()->fetchOne(
                "SELECT COUNT(*) FROM " . self::$_db_prefix . "catalog_product_category WHERE category_id = ?", 
                $categoryId
            );
        }
    }
    
    public function countBySite($siteId)
    {
        return $this->getAdapter()->fetchOne(
            "SELECT COUNT(DISTINCT ptc.product_id) FROM " . self::$_db_prefix . "catalog_category c
            INNER JOIN " . self::$_db_prefix . "catalog_product_category ptc ON c.id = ptc.category_id
            WHERE c.site_id = ?", $siteId
        );
    }
    
    /**
     * Retrieve count of uncategorized products
     * 
     * @return int
     */
    public function countByUncategorized()
    {       
        $query = "SELECT COUNT(p.id) 
            FROM " . self::$_db_prefix . 'catalog_product' . " AS p
            WHERE p.id <> ALL (
                SELECT product_id FROM " . self::$_db_prefix . "catalog_product_category AS ptc
            )
            OR p.id = ANY (
                SELECT product_id
                FROM " . self::$_db_prefix . "catalog_product_category AS ptc
                INNER JOIN " . self::$_db_prefix . "catalog_category AS c ON c.id = ptc.category_id
                WHERE c.lvl = 0
            )";
        
        return $this->getAdapter()->fetchOne($query);
    }
    
    public function getProductBestSeller($params = array())
    {
        $select = $this->getAdapter()->select();
        $select->from(
                array('op' => self::$_db_prefix . 'sales_order_product'),
                array ('product_id', 'product_name',
                'SUM(op.product_quantity) as ordered_qty')
            )
            ->joinInner(array('p' => self::$_db_prefix . 'catalog_product'),
                "p.id = op.product_id"
            )
            ->group('op.product_id')
            ->order('ordered_qty DESC');
        if (isset($params['categoryId'])) {
            $select->joinLeft(
                array('ptc' => self::$_db_prefix . 'catalog_product_category'), 
                "ptc.product_id = p.id",
                array()
            );
            $select->where($this->getAdapter()->quoteInto(
                'ptc.category_id = ?', $params['categoryId']
            ));
        }
        if ($params['siteId']) {
            $select->joinLeft(
                array('ptc2' => self::$_db_prefix . 'catalog_product_category'), 
                "ptc2.product_id = p.id",
                array()
            );
            $select->joinLeft(array('c' => self::$_db_prefix . 'catalog_category'), 
                "ptc2.category_id = c.id",
                array()
            );
            $select->where($this->getAdapter()->quoteInto(
                'c.site_id = ?', $params['siteId']
            ));
        }
        if (!empty($params['limit']))
            $select->limit($params['limit'], $params['start']);
        return $select->query()->fetchAll();
    }
    
    public function getViewed($params = array())
    {   
        $select = $this->getAdapter()->select();
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'),
            array('id', 'price', 'viewed')
        );
        $select->joinLeft(
            array('pd' => self::$_db_prefix . 'catalog_product_description'), 
            "pd.product_id = p.id AND pd.language_id = " . $params['languageId'],
            array('product_name' => 'name')
        );
        $select->where('p.viewed > 0');
        if (!empty($params['limit']))
            $select->limit($params['limit'], $params['start']);
        if (!empty($params['dir']))
            $select->order(' viewed ' . $params['dir']);
        else 
            $select->order('viewed DESC');
            
        if ($params['siteId']) {
            $select->joinLeft(
                array('ptc2' => self::$_db_prefix . 'catalog_product_category'), 
                "ptc2.product_id = p.id",
                array()
            );
            $select->joinLeft(
                array('c' => self::$_db_prefix . 'catalog_category'), 
                "ptc2.category_id = c.id",
                array()
            );
            $select->where($this->getAdapter()->quoteInto(
                'c.site_id = ?', $params['siteId']
            ));
        }    
        
        return $select->query()->fetchAll();
    }
    
    public function getStock($productId)
    {
        return Ecart::single('catalog/product_stock')->getStock($productId);
    }

    /**
     *
     * @return Ecart_Catalog_Model_Product_Stock
     */
    public function stock()
    {
        return Ecart::single('catalog/product_stock');
    }

    /**
     *
     * @param int $productId
     * @param int $variationId [optional]
     * @return float
     */
    public function getQuantity($productId, $variationId = null)
    {
        return Ecart::single('catalog/product_stock')
            ->getQuantity($productId, $variationId);
    }

    /**
     *
     * @param int $productId
     * @param float $quantity
     * @param int $variationId
     * @return mixed
     */
    public function setQuantity($productId, $quantity, $variationId = null)
    {
        return Ecart::single('catalog/product_stock')
            ->setQuantity($productId, $quantity, $variationId);
    }

    /**
     *
     * @param int $productId
     * @param float $quantity
     * @param int $variationId
     * @return bool
     */
    public function canAddToCart($productId, $quantity, $variationId = null)
    {
        return Ecart::single('catalog/product_stock')->canAddToCart(array(
            'product_id' => $productId,
            'quantity'   => $quantity,
            'product_variation_id' => $variationId
        ));
    }

    /**
     *
     * @param int $productId
     * @return array
     */
    public function getCategories($productId)
    {
        $categories = Ecart::single('catalog/product_category')
            ->getCategoriesByProductIds(array($productId));
        return isset($categories[$productId]) ?
            $categories[$productId] : array();
//        return $this->getAdapter()->fetchCol(
//            "SELECT category_id FROM " . self::$_db_prefix . "catalog_product_category " .
//            "WHERE product_id = ? ",
//            $productId
//        );
    }
    
    // GBASE IMPORT FUNCTION(s)
    public function getNextImportProductId(
        $categoryId, $currentProductId, $recursive)
    {
        $category = Ecart::single('catalog/category')
            ->find($categoryId)->current();
        if (!$category)
            return false;
        $select = $this->getAdapter()->select();
        
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'),
            array('p.id')
        );
        $select->join(
                array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                'p.id = ptc.product_id',
                array()
            )
            ->join(
                array('c' => self::$_db_prefix . 'catalog_category'),
                'c.id = ptc.category_id',
                array()
            )
            ->where('c.site_id = ?', $category->site_id);
            
        if ($recursive && $category->lvl != 0) {
            $select->where("c.lft BETWEEN $category->lft AND $category->rgt")
                ->where("c.rgt BETWEEN $category->lft AND $category->rgt");
        } elseif ($category->lvl != 0) {
            $select->where('c.id = ?', $category->id);
        }
        
        if (!empty(Ecart::session()->last_exported))
            $select->where('p.id > ?', $currentProductId);
            
        $select->order('p.id', 'ASC');
        
        return $this->getAdapter()->fetchOne($select);
    }

    /**
     *
     * @param array $productIds
     * @return array
     */
    public function getProductListByIds($productIds)
    {
        if (empty($productIds)) {
            return array();
        }
        
        if (!is_array($productIds)) {
            $productIds = array($productIds);
        }
        
        $select = $this->getAdapter()->select();
        $select->from(array('p' => self::$_db_prefix . 'catalog_product'), 'id')
            ->where($this->getAdapter()->quoteInto('p.id IN (?)', $productIds));

        $products = $this->getAdapter()->fetchAssoc($select);
        $this->load($products);

        Ecart::single('discount/discount')->fillDiscount($products);
        return $products;
    }
    
    /**
     * 
     * @return object
     * @param object $select     
     * @param string $order
     */
    protected function _setSort(&$select, $order = 'name', $dir = 'ASC')
    {
        if (!in_array(strtoupper($dir), array('ASC', 'DESC'))) {
            $dir = 'ASC';
        }
        switch ($order) {
            case 'name': 
                $select->order("pd.name {$dir}");
                break; 
            case 'price':
                //@todo also need use discount price
                $select->order("p.price {$dir}");
                break;
            case 'rate':
            default:
        }
        
        return $select;
    }
    
    /**
     *  Get product ids array by filter (used on product listing)
     * 
     *  @param  filters array()
     *  @return  array
     */ 
    public function getProductsByFilter(array $filters)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('p' => self::$_db_prefix . 'catalog_product'),
                array(new Zend_Db_Expr('SQL_CALC_FOUND_ROWS DISTINCT p.id'))
            )
            ->where('p.is_active = 1')
            // join for sorting products by name
            ->joinLeft(
                array('pd' => self::$_db_prefix . 'catalog_product_description'),
                'pd.product_id = p.id AND pd.language_id = ' . Ecart_Locale::getLanguageId(),
                array('name')
            );
            
        $this->setFilters($select, $filters);
        $this->_setSort($select, $filters['order'], $filters['dir']);
        
        $products = $this->getAdapter()->fetchAssoc($select);
        $count = Ecart::db()->fetchOne("SELECT FOUND_ROWS()");
        if (!$count) {
            return array('count' => 0);
        }
        
        $this->load($products);
        
        Ecart::single('discount/discount')->fillDiscount($products);
        
        $products['count'] = $count;
        return $products;
    }
    
    /**
     * Load full product information, including variation and modifiers
     * 
     * @param array $products Where keys are product ids
     * @return void
     */
    public function load(&$products)
    {
        if (!count($products)) {
            return;
        }
        /*
        $select = $this->getAdapter()->select();
        $select->from('p' => self::$_db_prefix . 'catalog_product')
        ->joinLeft(array('hu' => self::$_db_prefix . 'catalog_hurl'),
            'hu.key_id = p.id'
            array('product_id' => 'hu.key_id', 'hu.key_word')
        )
        ->joinLeft(array('pd' => self::$_db_prefix . 'catalog_product_description'), 
            'pd.product_id = p.id AND pd.language_id = ' . $langId,
            array('name', 'image_seo_name', 'description'))
        ->join(array('cpv' => self::$_db_prefix . 'catalog_product_variation'),
            'hu.key_id = cpv.product_id OR cpv.product_id IS NULL',
            array('variation_id' => 'id',
                'variation_price' => 'price', 'variation_price_type' => 'price_type',
                'variation_sku' => 'sku', 'variation_quantity' => 'quantity',
                'variation_weight' => 'weight', 'variation_weight_type' => 'weight_type',

            )
        )
        ->joinLeft(array('cpa' => self::$_db_prefix . 'catalog_product_attribute'),
            'hu.key_id = cpa.product_id AND (cpa.product_variation_id = 0 OR cpa.product_variation_id = cpv.id)',
            array('product_option_id', 'product_option_value_id', 
                'attribute_price' => 'price', 'attribute_price_type' => 'price_type',
                'attribute_weight' => 'weight', 'attribute_weight_type' => 'weight_type',
                'modifier', 'product_variation_id', 'attribute_id' => 'id'
            )
        )
        ->where("hu.key_id IN(?) AND hu.key_type='p'", array_keys($products));
        */
        
        $select = "SELECT p.*," 
                . "`hu`.`key_id` AS `product_id`, `hu`.`key_word`," 
                . "`cpv`.`id` AS `variation_id`, `cpv`.`price` AS `variation_price`, "
                . "`cpv`.`price_type` AS `variation_price_type`, " 
                . "`cpv`.`sku` AS `variation_sku`, "
                . "`cpv`.`quantity` AS `variation_quantity`, "
                . "`cpv`.`weight` AS `variation_weight`, "
                . "`cpv`.`weight_type` AS `variation_weight_type`, "
                . "`cpa`.`product_option_id`, `cpa`.`product_option_value_id`, "
                . "`cpa`.`price` AS `attribute_price`, "
                . "`cpa`.`price_type` AS `attribute_price_type`, "
                . "`cpa`.`weight` AS `attribute_weight`, "
                . "`cpa`.`weight_type` AS `attribute_weight_type`, "
                . "`cpa`.`modifier`, `cpa`.`product_variation_id`, "
                . "`cpd`.`name`, `cpd`.`image_seo_name`,`cpd`.`description`,`cpd`.`short_description`, "
                . "`cpa`.`id` AS `attribute_id` "
                . " FROM `" . self::$_db_prefix . "catalog_product` AS `p` "
                . " LEFT JOIN `" . self::$_db_prefix . "catalog_hurl` AS `hu` "
                . " ON p.id = hu.key_id"
                . " LEFT JOIN `" . self::$_db_prefix . "catalog_product_variation` AS `cpv` "
                . " ON hu.key_id = cpv.product_id OR cpv.product_id IS NULL "
                . " LEFT JOIN `" . self::$_db_prefix . "catalog_product_attribute` AS `cpa` "
                . " ON hu.key_id = cpa.product_id AND (cpa.product_variation_id = 0 "
                . "  OR cpa.product_variation_id = cpv.id) "
                . " LEFT JOIN `" . self::$_db_prefix . "catalog_product_description` AS `cpd` "
                . " ON hu.key_id = cpd.product_id AND cpd.language_id = " . Ecart_Locale::getLanguageId() . " "
                . $this->getAdapter()->quoteInto(
                    "WHERE (hu.key_id IN(?) AND hu.key_type='p')", array_keys($products)
                );
                
        $processed = array();
        $product_keys = array('');
        foreach ($this->getAdapter()->fetchAll($select) as $row) {
            if (!isset($processed[$row['product_id']])) {
                $processed[$row['product_id']] = true;
                $products[$row['product_id']]['is_saleable'] = $row['quantity'] > 0;
                foreach ($row as $key => $value) {
                    if (in_array($key, array('id','manufacturer_id','quantity',
                            'sku','image_base','image_listing','image_thumbnail',
                            'price','date_available','weight','is_active','ordered',
                            'created_on','modified_on','tax_class_id','viewed',
                            'new_from','new_to','product_id','key_word','name',
                            'image_seo_name','description', 'short_description'))) {
                        
                        $products[$row['product_id']][$key] = $value;
                    }
                }
            }
            if (!isset($products[$row['product_id']]['variation'][$row['variation_id']])) {
                $sku = $row['variation_id'] /*variationId != 0*/ ? 
                    $row['variation_sku'] : $row['sku'];
                    
                $quantity = $row['variation_id'] /*variationId != 0*/? 
                    $row['variation_quantity'] : $row['quantity'];
                    
                $products[$row['product_id']]['variation'][$row['variation_id']] =  array(
                    'id'            => $row['variation_id'],
                    'price_amount'  => $row['variation_price'],
                    'price_type'    => $row['variation_price_type'],
                    'sku'           => $sku,
                    'quantity'      => $quantity,
                    'weight_amount' => $row['variation_weight'],
                    'weight_type'   => $row['variation_weight_type'],
                );
                
                if (!$products[$row['product_id']]['is_saleable']) {
                    $products[$row['product_id']]['is_saleable'] = $quantity > 0;
                }
            }
            if ($row['product_variation_id']) {
                $products[$row['product_id']]['variation']
                        [$row['variation_id']]['option']
                            [$row['product_option_id']]
                                = $row['product_option_value_id'];
            }

            if ($row['modifier'] 
                && !isset($products[$row['product_id']]['modifier'][$row['attribute_id']])) {

                $products[$row['product_id']]['modifier']
                    [$row['attribute_id']] = array(
                        'option_id'         => $row['product_option_id'],
                        'option_value_id'   => $row['product_option_value_id'],
                        'price_amount'      => $row['attribute_price'],
                        'price_type'        => $row['attribute_price_type'],
                        'weight_amount'     => $row['attribute_weight'],
                        'weight_type'       => $row['attribute_weight_type'],
                    );
            }
        }
    }
    
    /**
     * Return price filters
     * 
     * @return mixed
     */
    public function getFiltersByPrice(array $filters)
    {
        $select = $this->getAdapter()->select();
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'),
            array('COUNT(p.price) as cnt, MAX(p.price) as price_max, MIN(p.price) as price_min')
        );
        $this->setFilters($select, $filters);
        
        $row = $this->getAdapter()->fetchRow($select);
        if ($row['cnt'] > 1) {
            //Return rounded number, example: 80->10, 120->100, 895->100, 1024->1000
            $roundTo = pow(10, strlen((string) floor($row['price_max'] - $row['price_min'])) - 1);
            $select->reset();
            $select->from(
                array('p' => self::$_db_prefix . 'catalog_product'),
                array('COUNT(DISTINCT p.id) as cnt', new Zend_Db_Expr("floor(p.price / $roundTo) * $roundTo as price_group"))
            );
            $this->setFilters($select, $filters);
            $select->group('price_group');
            $select->order('p.price');
            $priceGroups = $this->getAdapter()->fetchAll($select);
            if (sizeof($priceGroups) < 2) {
                return null;
            }
            return array(
                'roundTo' => $roundTo,
                'groups' => $priceGroups
            );
        }
        return null;
    }
    
    /**
     * @param  array
     * @return mixed
     */
    public function getFiltersByManufacturer(array $filters)
    {
        $select = $this->getAdapter()->select();
        
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'),
            array('COUNT(DISTINCT p.id) as cnt')
        );
        $this->setFilters($select, $filters);
        $select->join(array('m' => self::$_db_prefix . 'catalog_product_manufacturer'), 
                'p.manufacturer_id = m.id', 
                array('id', 'name', 'image')
            )
            ->join(array('cpmt' => self::$_db_prefix . 'catalog_product_manufacturer_title'),
                'm.id = cpmt.manufacturer_id AND cpmt.language_id = ' . Ecart_Locale::getLanguageId(),
                'title'
            )
            ->join(array('ch' => self::$_db_prefix . 'catalog_hurl'),
                "ch.key_id = m.id AND ch.key_type = 'm' AND ch.site_id = " . Ecart::getSiteId(),
                array('url' => 'key_word')
            )
            ->group('m.id')
            ->order('cpmt.title');
            
        $manufacturers = $this->getAdapter()->fetchAll($select);
        if (!count($manufacturers)) {
            return null;
        }
        
        return $manufacturers;
    }
    
    /**
     * Возвращает список фильтрова для атрибутов, пример запроса ниже
     * SELECT COUNT(*) as cnt,
     *        pa.product_option_id, pa.product_option_value_id,
     *        pot.name as option_name, pov.name as value_name
     * FROM product p
     * INNER JOIN product_attribute pa ON pa.product_id = p.id
     * INNER JOIN product_option_text pot ON pa.product_option_id = pot.product_option_id
     * INNER JOIN product_option_value pov ON pa.product_option_value_id = pov.id
     * WHERE pot.language_id = 1 AND pov.language_id = 1
     * GROUP BY pa.product_option_value_id;
     * 
     * @return mixed
     */
    public function getFiltersByAttribute(array $filters)
    {
        $select = $this->getAdapter()->select();
        $select->from(
            array('p' => self::$_db_prefix . 'catalog_product'),
            array('cnt' => 'COUNT(distinct p.id, `pa`.`product_option_id`,`pa`.`product_option_value_id`) as cnt')
        );
        
        //$this->_fillFiltersConditions($select, array('attributes')); ???
        $select->join(
            array('pa' => self::$_db_prefix . 'catalog_product_attribute'),
            'p.id = pa.product_id',
            array('product_option_id', 'product_option_value_id')
        )->join(
            array('po' => self::$_db_prefix . 'catalog_product_option'),
            'pa.product_option_id = po.id',
            array()
        )->join(
            array('pot' => self::$_db_prefix . 'catalog_product_option_text'),
            'pa.product_option_id = pot.product_option_id',
            array('option_name' => 'name')
        )->join(
            array('povt' => self::$_db_prefix . 'catalog_product_option_value_text'),
            'pa.product_option_value_id = povt.product_option_value_id',
            array('value_name' => 'name')
        )->where('po.filterable = 1')
        ->where('pot.language_id = ?', Ecart::getLanguageId())
        ->where('povt.language_id = ?', Ecart::getLanguageId())
        ->group(array('pa.product_option_id', 'pa.product_option_value_id'));
        
        if (isset($filters['attributes']) && count($filters['attributes'])) {
            $select->where('pa.product_option_id NOT IN (?)', array_keys($filters['attributes']));
        }
        
        $this->setFilters($select, $filters);
        $attributes = $this->getAdapter()->fetchAll($select);
        if (!count($attributes)) {
            return null;
        }
        
        $filters = array();
        foreach ($attributes as $attribute) {
            if (!isset($filters[$attribute['product_option_id']])) {
                $filters[$attribute['product_option_id']] = array();
            }
            
            $filters[$attribute['product_option_id']][] = $attribute;
        }
        return $filters;
    }
    
    /**
     * 
     * @param select Zend_Db_Select or Zend_Db_Table_Select
     * @param array ( 'siteId' => 1 int
     *                  'categoryIds' => array(1,2,4),
     *                  'product_id' => int,
     *                  'product_name' => string,
     *                  'manufacturerId' = int
     *                  'price' => array('from' => int, 'to' => int)
     *                  'attributes' => array()
     *                  'page' => int
     *                  'limit' => int
     *                      
     *  )
     */
    public function setFilters(&$select, array $filters) 
    {
        $today = Ecart_Date::now()->toPhpFormat()->toString('Y-m-d');
        $select->where('p.date_available IS NULL OR p.date_available <= ?', $today);
        // filter by site
        $select->join(
            array('ptc' => self::$_db_prefix . 'catalog_product_category'),
            'p.id = ptc.product_id',
            array()
        );
        $select->join(
            array('c' => self::$_db_prefix . 'catalog_category'),
            'ptc.category_id = c.id',
            array()
        )->where('c.cat_status = "enabled"');
        // filter by product_id
        if (isset($filters['product_id'])) {
            $select->where('p.id = ?', $filters['product_id']);
        }
//        // filter by product_name
//        if (isset($filters['product_name'])) {
//            $select->where("pd.name LIKE '%{$filters['product_name']}%'");
//        }
        // filter by siteId
        if (isset($filters['siteId'])) {
            $select->where('c.site_id = ' . intval($filters['siteId']));
        }
        // filter by category
        if (isset($filters['categoryIds'])) {
            if ( !is_array($filters['categoryIds'])) {
                $filters['categoryIds'] = array($filters['categoryIds']);
            }
            if (count($filters['categoryIds'])) {
                $select->where('ptc.category_id IN (?)', $filters['categoryIds']);
            }
        }
        // filter by manufacturer
        if (isset($filters['manufacturerId'])) {
            $select->where('p.manufacturer_id = ?', $filters['manufacturerId']);
        }
        // by price
        if (isset($filters['price']['from'])) {
            $select->where('p.price >= ?', $filters['price']['from']);
        }
        
        if (isset($filters['price']['to'])) {
             $select->where('p.price <= ?', $filters['price']['to']);
        }
        /*
         *  
         * Если уже есть фильтр по атрибутам, то пропускаем эти значения атрибутов
         * и выбираем атрибуты только для тех продуктов которые имеют атрибути из
         * существующего фильтра по атрибутам :). Например если у нас уже есть фильтр по
         * синему цвету то мы пропускаем опцию цвет, а вибираем атрибуты размера(например)
         * только для продуктов которые имеют синий цвет
         *
         *  by attributes
         */
        if (isset($filters['attributes']) && count($filters['attributes'])) {
            $aId = $filters['attributes'];
            $aIdStack = array();
            $i = 0;
            foreach ($aId as $oId => $id) {
                $id = (int) $id;
                $select->join(
                    array("pac$i" => self::$_db_prefix . 'catalog_product_attribute'),
                    "pac$i.product_id = p.id",
                    array()
                )
                ->where("pac$i.product_option_id = '$oId'")
                ->where("pac$i.product_option_value_id = '$id'");
                if ($i > 0) {
                    $where = "if(pac$i.product_variation_id=0, 1, if (0 < " . implode('+', $aIdStack).", pac$i.product_variation_id IN(".implode(',', $aIdStack)."), 1))";
                    $select->where($where);
                }
                $aIdStack[] = 'pac' . $i . '.product_variation_id';
                ++$i;
            }
        }
        
        if (isset($filters['start']) && isset($filters['limit'])) {
            $select->limit($filters['start'], $filters['limit']);
        } elseif (isset($filters['page']) && isset($filters['limit'])) {
            $select->limitPage($filters['page'], $filters['limit']);
        } elseif (isset($filters['limit'])) {
            $select->limit($filters['limit']);
        }
        return $select;   
    }
    
    // Csv import-export functions
    /**
     * 
     * @return Zend_Db_Table_Rowset
     * 
     * @param integer $minId[required] exclusive bound of product_id
     * @param integer $limit[optional] max count of rowset
     * @param array $filters[optional] example:
     *     array(
     *         'status' => 0 - disabled, 1 - enabled, 2 - all,
     *         'stock' => 0 - out of stock, 1 - in stock, 2 - all,
     *         'name' => 'Cellphone',
     *         'sku' => '10002',
     *         'site' => '1,2' site ids, 'all' - all sites
     *     )
     */
    public function getProductSet($minId, $limit = 1, array $filters = null) 
    {
        if (!is_numeric($minId))
            return array();
            
        $select = $this->select()->distinct()
            ->from(array('p' => self::$_db_prefix . 'catalog_product'))
            ->where('p.id > ' . $minId)
            ->order('p.id ASC')
            ->limit($limit);
        
        if (null !== $filters) {
            if (isset($filters['status']) && $filters['status'] != 2 && is_numeric($filters['status'])) {
                $select->where('p.is_active = ' . intval($filters['status']));
            }
            if (isset($filters['stock']) && $filters['stock'] != 2 && is_numeric($filters['stock'])) {
                $select->join(array('ps' => self::$_db_prefix . 'catalog_product_stock'),
                    'p.id = ps.product_id',
                    array())
                    ->where('ps.in_stock = ' . intval($filters['stock']));
            }
            if (isset($filters['site']) && $filters['site'] != 'all' && trim($filters['site'], ' ,') != '') {
                $site_count = count(Ecart_Collect_Site::collect());
                $filter_sites = explode(',', trim($filters['site'], ' ,'));
                $filters_site_count = count($filter_sites);
                if ($site_count != $filters_site_count) {
                    $select->join(array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                        'p.id = ptc.product_id',
                        array())
                        ->join(array('c' => self::$_db_prefix . 'catalog_category'),
                        'ptc.category_id = c.id',
                        array())
                        ->where('c.site_id IN (?)', $filter_sites);
                }
            }
            if (isset($filters['name']) && $filters['name'] != '') {
                $select->join(array('pd' => self::$_db_prefix . 'catalog_product_description'),
                    'p.id = pd.product_id',
                    array())
                    ->where('pd.name LIKE ?', $filters['name'].'%');
            }
            if (isset($filters['sku']) && $filters['sku'] != '') {
                $select->where('p.sku LIKE ?', $filters['sku'].'%');
            }
            /* price filter */
            if (isset($filters['price_from']) && $filters['price_from'] != '' && is_numeric($filters['price_from'])) {
                $select->where('p.price >= ?', $filters['price_from']);
            }
            if (isset($filters['price_to']) && $filters['price_to'] != '' && is_numeric($filters['price_to'])) {
                $select->where('p.price <= ?', $filters['price_to']);
            }
            /* qty filter */
            if (isset($filters['qty_from']) && $filters['qty_from'] != '' && is_numeric($filters['qty_from'])) {
                $select->where('p.quantity >= ?', $filters['qty_from']);
            }
            if (isset($filters['qty_to']) && $filters['qty_to'] != '' && is_numeric($filters['qty_to'])) {
                $select->where('p.quantity <= ?', $filters['qty_to']);
            }
        }
        
        return $this->fetchAll($select);
    }
    
    /**
     * @param  params = array('siteId' => int , 
     *                        'currentDate' =>  Ecart_Date::now()->toPhpFormat()->toString("Y-m-d"),
     *                        'categoryId' => int 
     *                        'limit' => int
     * )
     * @return array of ids
     */
    public function getNewProductIds(array $params)
    {
        $select = $this->getAdapter()->select();
        $select->distinct()->from(
                    array('p' => self::$_db_prefix . 'catalog_product'),
                    array('id')
                )
               ->where('p.new_from IS NULL OR p.new_from <= ?', $params['currentDate'])
               ->where('p.new_to IS NULL OR p.new_to >= ?', $params['currentDate'])
               ->where('p.new_from IS NOT NULL OR p.new_to IS NOT NULL')
               ->where('p.date_available IS NULL OR p.date_available <= ?', $params['currentDate'])
               ->where('p.is_active = 1')
               ->order(new Zend_Db_Expr("RAND()"))
               ->limit($params['limit']);
        
        $select->join(
            array('ptc' => self::$_db_prefix . 'catalog_product_category'),
            'p.id = ptc.product_id',
            array()
        )->join(
            array('c' => self::$_db_prefix . 'catalog_category'),
            'ptc.category_id = c.id',
            array()
        )->where('c.site_id = ' . intval($params['siteId']));
        
        if ($disabledCategories = Ecart::single('catalog/category')->getDisabledIds()) {
            $select->where('c.id NOT IN (?)', $disabledCategories);
        }
        
        if (null !== $params['categoryId']) {
            $select->where('ptc.category_id = ' . intval($params['categoryId']));
        }
        
        return $this->getAdapter()->fetchCol($select);
    }
    
    public function getPopularProductIds(array $params)
    {
        $select = $this->getAdapter()->select();
        $select->distinct()->from(
                array('p' => self::$_db_prefix . 'catalog_product'),
                array('id')
            )
            ->join(array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                'p.id = ptc.product_id',
                array())
            ->join(array('c' => self::$_db_prefix . 'catalog_category'),
                'ptc.category_id = c.id',
                array())
            ->where('c.site_id = ' . intval($params['siteId']))
            ->where('p.date_available IS NULL OR p.date_available <= ?', $params['currentDate'])
            ->where('p.is_active = 1')
            ->order('p.viewed DESC')
            ->limit($params['limit']); 
        
        if ($disabledCategories = Ecart::single('catalog/category')->getDisabledIds()) {
            $select->where('c.id NOT IN (?)', $disabledCategories);
        }
        
        if (null !== $params['categoryId']) {
            $select->where('ptc.category_id = ' . intval($params['categoryId']));
        }
        
        return $this->getAdapter()->fetchCol($select);
    }
    
    public function getBestsellerProductIds(array $params)
    {
        $select = $this->getAdapter()->select();
        $select->distinct()->from(
                array('p' => self::$_db_prefix . 'catalog_product'),
                array('id')
            )
            ->join(array('ptc' => self::$_db_prefix . 'catalog_product_category'),
                'p.id = ptc.product_id',
                array())
            ->join(array('c' => self::$_db_prefix . 'catalog_category'),
                'ptc.category_id = c.id',
                array())
            ->where('c.site_id = ' . intval($params['siteId']))
            ->where('p.date_available IS NULL OR p.date_available <= ?', $params['currentDate'])
            ->where('p.is_active = 1')
            ->order('p.ordered DESC')
            ->limit($params['limit']); 
        
        if ($disabledCategories = Ecart::single('catalog/category')->getDisabledIds()) {
            $select->where('c.id NOT IN (?)', $disabledCategories);
        }
        
        if (null !== $params['categoryId']) {
            $select->where('ptc.category_id = ' . intval($params['categoryId']));
        }
        
        return $this->getAdapter()->fetchCol($select);
    }
    
    public function getFullDescriptionByProductIds($productIds, $languageId = null)
    {
        if (null === $languageId) {
            $languageId = Ecart::getLanguageId();
        }
        $rows = $this->find(array_values($productIds));
        
        $products = array();
        foreach ($rows as $row) {
            $products[$row->id] = $row->toArray();
            $products[$row->id]['description'] = $row->getDescription();
            $products[$row->id]['options'] = array();
        }
        
        $whereIn = $this->getAdapter()->quoteInto('?', array_values($productIds));
        $query = 'SELECT pa.product_id, po.id as option_id, pot.name as name, ' 
                      . 'if (povt.name !=\'\', povt.name, pav.attribute_value) as value '
               . "FROM " . self::$_db_prefix . "catalog_product_attribute pa "
               . "INNER JOIN " . self::$_db_prefix . "catalog_product_option po ON po.id = pa.product_option_id  AND po.comparable = 1 "
               . "INNER JOIN " . self::$_db_prefix . "catalog_product_option_text pot ON pot.product_option_id = pa.product_option_id "
               . "LEFT JOIN  " . self::$_db_prefix . "catalog_product_option_value_text povt ON povt.product_option_value_id = pa.product_option_value_id AND "
                                                            . 'povt.language_id = :langId '
               . "LEFT JOIN " . self::$_db_prefix . "catalog_product_attribute_value pav ON pav.product_attribute_id = pa.id AND " 
                                                        . '(pav.language_id = :langId OR pav.language_id is null) ' 
               . "WHERE pa.product_id IN($whereIn) AND pot.language_id = :langId "
               . 'ORDER BY po.sort_order';
        $stmt = $this->getAdapter()->query($query, array(
            'langId' => $languageId
        ));
        while (($row = $stmt->fetch(Zend_Db::FETCH_OBJ))) {
            if (!isset($products[$row->product_id]['options'][$row->option_id])) {
                $products[$row->product_id]['options'][$row->option_id] = array(
                    'name' => $row->name, 
                    'values' => array()
                );
            }
            
            if (!in_array($row->value, $products[$row->product_id]['options'][$row->option_id]['values']))
                $products[$row->product_id]['options'][$row->option_id]['values'][] = $row->value; 
        }
        $this->load($products);
        return $products;
    }
    
    /**
     * Retrieve product row by url
     * 
     * @param string $url
     * @return mixed int|bool
     */
    public function getByUrl($url)
    {
        return $this->fetchRow($this->select()
            ->from(array('p' => self::$_db_prefix . 'catalog_product'))
            ->join(array('ch' => self::$_db_prefix . 'catalog_hurl'),
                'p.id = ch.key_id',
                array())
            ->where('ch.key_type = ?', 'p')
            ->where('ch.site_id = ?', Ecart::getSiteId())
            ->where('ch.key_word = ?', $url));
    }
}