<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Checkout_Model_Cart extends Ecart_Db_Table
{
    protected $_name = 'checkout_cart';

    /**
     *
     * @var int
     */
    private $_cartId;
    
    public $total;
        
    /**
     * Return customer cart
     *
     * @param integer $customerId
     * @return Ecart_Checkout_Model_Cart
     */
    public function getCustomerCart($customerId)
    {
        $id = $this->getIdByCustomerId($customerId);
        if (!$id) {
           return null;
        }
        $this->setCartId($id);

        return $this;
    }
    
    /**
     *
     * @return bool
     */
    public function isValid() 
    {
        if (!isset(Ecart::session()->cartId) 
            || null === Ecart::session()->cartId) {

            return false;
        }
        if (!$this->hasId(Ecart::session()->cartId)) {
            return false;
        }
        return true;
    }
    
    /**
     * Init
     *
     * @return int
     */
    public function initCartId()
    {
        if ($this->isValid()) {
            $cartId = Ecart::session()->cartId;
        } elseif ($customerId = Ecart::getCustomerId()) {
            $cartId = $this->getIdByCustomerId($customerId);
        } 
        
        if (!isset($cartId) || !$cartId) {
            $cartId = $this->insert(array(
                'site_id' => Ecart::getSiteId(),
                'customer_id' => $customerId ?
                    $customerId : new Zend_Db_Expr('NULL')
            ));
        }
        
        $this->setCartId($cartId);
  
        return $cartId;
    }


    public function unsetCartId() 
    {
        Ecart::session()->cartId = null;
        $this->_cartId = null;
    }

    /**
     *
     * @param int $id
     * @return void
     */
    public function setCartId($id)
    {
        Ecart::session()->cartId = $id;
        $this->_cartId = $id;
    }
    
    /**
     * @return int
     */
    public function getCartId()
    {
        if (null === $this->_cartId) {
            $this->initCartId();
        }
        return $this->_cartId;
    }
    
    /**
     * Wraper on setCartId
     *
     * @return int
     */
    public function getId()
    {
        return $this->getCartId();
    }
    
     /**
      *
      * @param int $productId
      * @param array $attributes
      * @param int $shoppingCartId [option]
      * @return mixed array|bool
      */
    private function _getClon($productId, $attributes, $shoppingCartId = null)
    {
        if (null === $shoppingCartId) {
            $shoppingCartId = $this->getCartId();
        }
        $result = Ecart::single('checkout/cart_product')->getAttributes(
            $shoppingCartId, $productId
        );
        $items = array();
        while ($row = $result->fetch()) {
            if (!isset($items[$row['id']])) {
                $items[$row['id']] = array(
                    'quantity' => $row['quantity'],
                    'attributes' => array()
                );
            }
            if (!isset($row['attrId'])) {
                continue;
            } 
            $items[$row['id']]['attributes'][$row['attrId']] =
                $row['product_attribute_value'];
        }
        foreach ($items as $itemId => $item) {
            if ((count($item['attributes']) == count($attributes)) &&
                !sizeof(array_diff_assoc($item['attributes'], $attributes))) 
            {
                return array('id' => $itemId, 'quantity' => $item['quantity']); 
            }
        }
        return false;
    }

    /**
     *
     * @param array $modifiers
     * @param int $optionId
     * @param mixed $valueId [optional]
     * @return int
     */
    private function _findAttributeIdByModifier(
        $modifiers, $optionId, $valueId = null)
    {
        foreach ($modifiers as $item) {
            if ($item['id'] != $optionId) {
                continue;
            }
            if (null != $valueId && count($item['values'])) {
                foreach ($item['values'] as $value) {
                    if ($valueId != $value['id']) {
                        continue;
                    }
                    return $value['attribute_id'];
                }
            } elseif ($item['type'] == Ecart_Catalog_Model_Product_Option::TYPE_STRING) {
                return $item['attribute_id'];
            } elseif ($item['type'] == Ecart_Catalog_Model_Product_Option::TYPE_TEXTAREA) {
                return $item['attribute_id'];
            } elseif (!$valueId) {
                return $item['attribute_id'];
            }

        }
        return 0;
    }

    /**
     *
     * @param array $variations
     * @param int $optionId
     * @param int $valueId
     * @return int
     */
    private function _findAttributeIdByVariation(
        &$variations, $optionId, $valueId)
    {
        foreach ($variations as $item) {
            if ($optionId == $item['product_option_id'] && 
                $valueId == $item['product_option_value_id'])
                return $item['id'];
        }
        return 0;
    }
    
    /**
     * Add new product or product variation to shopping cart
     *
     * @param int $productId
     * @param array $modifiers
     * @param array $vAttributes
     * @param array $quantity [optional]
     * @return mixed int|bool
     */
    public function add($productId, $modifiers, $vAttributes, $quantity = 1)
    {
        $tableProduct = Ecart::single('catalog/product');
        $product = $tableProduct->find($productId)->current();
        if (!$product) {
            Ecart::message()->addError(
                Ecart::translate('catalog')->__(
                    'Product not found'
                )
            );
            return false;
        }
        
        $stock = Ecart::single('catalog/product_stock')
            ->find($productId)->current();

        if ($stock->decimal) {
            $quantity = floor($quantity);
        }

        $row = array(
            'shopping_cart_id' => $this->getCartId(),
            'product_id' => $productId,
            'quantity' => $quantity,
            'final_price' => 0,
            'final_weight' => 0,
            'product_variation_id' => 0
        );
        
        /* Check for variation exists */
        foreach ($vAttributes as $id => $value) {
            if (!$value) {
                unset($vAttributes[$id]);
            }
        }
        
        if (count($vAttributes)) {
            $variationId = $product->getVariationIdByAttributes($vAttributes);
            if (!$variationId) {
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        'Fill all variation options please'
                    )
                );
                return false;
            }
            $row['product_variation_id'] = $variationId;
        }
        
        if (!Ecart::single('catalog/product_stock')->canAddToCart($row, true)) {
            return false;
        }
        
        $modifiersAll = $product->getModifiers();
        
        $countRequiredModifiers = 0;
        foreach ($modifiersAll as $modifier) {
            if ('3' === $modifier['type']) {continue;}
            $countRequiredModifiers++;
        }
        
        $countRecievedModifiers = 0;
        if (isset($modifiers['value'])) {
            $countRecievedModifiers += count($modifiers['value']);
        }
        if (isset($modifiers['id'])) {
            $countRecievedModifiers += count($modifiers['id']);
        }
        
        if ($countRecievedModifiers < $countRequiredModifiers) {
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    'Set required attributes'
                )
            );
            return false;
        }
        // find modifiers attribute ids
        $attributes = array();
        if (isset($modifiers['value'])) {
            foreach ($modifiers['value'] as $optionId => $value) {
                if ($value == '') {
                    continue; //fix textarea and text option type
                }
                $id = $this->_findAttributeIdByModifier(
                    $modifiersAll, $optionId
                );
                if (!$id) {
                    Ecart::message()->addError(
                        Ecart::translate('checkout')->__(
                            'Invalid modifier value recieved'
                        )
                    );
                    return false;
                }
                $attributes[$id] = $value;
            }
        }
        
        if (isset($modifiers['id'])) {
            foreach ($modifiers['id'] as $optionId => $valueIds) {
                if (!is_array($valueIds)) {
                    $valueIds = array($valueIds);
                }
                foreach ($valueIds as $vId) {
                    $id = $this->_findAttributeIdByModifier(
                        $modifiersAll, $optionId, $vId
                    );
                    if (!$id) {
                        Ecart::message()->addError(
                            'Invalid modifier id recieved'
                        );
                        return false;
                    }
                    $attributes[$id] = '';
                }
            }
        }
        // find variation attributes ids
        if (!empty($variationId)) {
            $variationAttrsAll = $product->getAttributesByVariationId(
                $variationId
            );
        } else {
            $variationAttrsAll =  array();
        }
        foreach ($vAttributes as $optionId => $valueId) {
            $id = $this->_findAttributeIdByVariation(
                $variationAttrsAll, $optionId, $valueId
            );

            if (!$id) {
                Ecart::message()->addError('Invalid variation recieved');
                return false;
            }
            $attributes[$id] = '';
        }
        
        Ecart::message()->addSuccess(
            Ecart::translate('checkout')->__(
                'Product was successfully added to your shopping cart'
            )
        );
        
        // Check for clon exists
        if ($clon = $this->_getClon($productId, $attributes)) {
            $this->updateItem($clon['id'], $clon['quantity'] + $quantity);
            return true;
        }
        
        // Insert product
        $row['final_price']  = $product->getPrice(array_keys($attributes));
        $row['final_weight'] = $product->getWeight(array_keys($attributes));
        $scItemId = Ecart::single('checkout/cart_product')->insert($row);
        
        // Insert attributes for this product
        $cartAttributeTable = Ecart::single('checkout/cart_product_attribute');
        foreach ($attributes as $attrId => $value) {
            $cartAttributeTable->insert(array(
                'shopping_cart_product_id' => $scItemId,
                'product_attribute_id' => $attrId,
                'product_attribute_value' => $value
            ));
        }
        
        Ecart::dispatch('checkout_cart_add_product_success', array(
            'product' => $product,
            'attributes' => $attributes,
            'quantity' => $quantity,
            'cart_product_id' => $scItemId
        ));
        
        return $scItemId;
    }
    
    /**
     * Updates quantity of cart product row.
     * !$itemId in not a product_id!
     * 
     * @param int $itemId Id of cart product row
     * @param int $quantity
     * @return mixed bool|void
     */
    public function updateItem($itemId, $quantity)
    {
        $item = Ecart::single('checkout/cart_product')->find($itemId)->current();
        
        if (!$product = Ecart::single('catalog/product')
                ->find($item->product_id)->current()) {
            
            $this->deleteItem($itemId);
            Ecart::message()->addError(
                Ecart::translate('checkout')->__(
                    "Product '%s' is not found in stock. product_id = %s",
                    $product->sku,
                    $item->product_id
                )
            );
            return false;
        }
        
        if ($quantity == 0) {
            $this->deleteItem($itemId);
            return true;
        }
        
        if (!$product->canAddToCart($quantity, $item->product_variation_id)) {
            if ($quantity = $product->getQuantity($item->product_variation_id, true)) {
                
                $item->quantity = $quantity;
                $item->save();
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        "Only %s item(s) of '%s' are available", 
                        $quantity,
                        $product->sku
                    )
                );
            } else {
                $this->deleteItem($itemId);
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        "Product '%s' is out of stock",
                        $product->sku
                    )
                );
            }
            return false;
        }
        
        $stock = Ecart::single('catalog/product_stock')
            ->find($item->product_id)->current();
        
        if ($stock->decimal) {
            $quantity = floor($quantity);
        }
        
        Ecart::dispatch('checkout_cart_update_product_success', array(
            'product' => $product,
            'quantity' => $quantity,
            'cart_product_id' => $itemId
        ));
        
        $item->quantity = $quantity;
        $item->save();
    }
    
    /**
     * 
     * @param int $itemId Id of cart_product_row (it's not a product_id)
     */
    public function deleteItem($itemId)
    {
        Ecart::dispatch('checkout_cart_remove_product_success', array(
            'cart_product_id' => $itemId
        ));
        return Ecart::single('checkout/cart_product')->delete(
            $this->getAdapter()->quoteInto('id = ?', $itemId)
        );
    }
    
    /**
     * Returns product list
     *
     * @return array mixed 
     */
    public function getProducts($shoppingCartId = null)
    {
        if (null === $shoppingCartId) {
            $shoppingCartId = $this->getCartId();
        }
        $result = Ecart::single('checkout/cart_product')
            ->getProducts($shoppingCartId);
        $products = array();
        while ($row = $result->fetch()) {
            if (!isset($products[$row['id']])) {
                $products[$row['id']] = $row;
                $products[$row['id']]['price_final'] = (float) $row['final_price'];
                $products[$row['id']]['decimal'] = (bool) $row['decimal'];
                $products[$row['id']]['attributes']  = array();
            }
            if (!isset($row['scpaId'])) {
                continue;
            }
            $attributtes = array();
            if (in_array(
                    $row['option_type'],
                    array(Ecart_Catalog_Model_Product_Option::TYPE_SELECT))) {

                $attributtes['name'] = $row['option_name'];
                $attributtes['value'] = $row['value_name'];
            } elseif (in_array(
                    $row['option_type'],
                    array(Ecart_Catalog_Model_Product_Option::TYPE_RADIO))) {

                $attributtes['name'] = $row['option_name'];
                $attributtes['value'] = $row['value_name'];
            } elseif (in_array(
                    $row['option_type'],
                    array(Ecart_Catalog_Model_Product_Option::TYPE_CHECKBOX))) {

                $attributtes['name'] = $row['option_name'];
                $attributtes['value'] = isset($row['value_name']) ?
                    $row['value_name'] : Ecart::translate('checkout')->__('Checked');
            } else {
                $attributtes['name'] = $row['option_name'];
                $attributtes['value'] = $row['product_attribute_value'];
            }
            $products[$row['id']]['attributes'][$row['scpaId']] = $attributtes;
        }
        return $products;
    }
    
    /**
     * Validates cart content. 
     * Write error messages to Ecart_Message
     * 
     * @return boolean
     */
    public function validateContent()
    {
        $isValid = true;
        foreach ($this->getProducts() as $cartProductRow) {
            if (!$product = Ecart::single('catalog/product')
                    ->find($cartProductRow['product_id'])->current()) {
                
                $isValid = false;
                $this->deleteItem($cartProductRow['id']);
                Ecart::message()->addError(
                    Ecart::translate('checkout')->__(
                        "Product '%s' is not found in stock. product_id = %s",
                        $cartProductRow['name'],
                        $cartProductRow['product_id']
                    )
                );
                continue;
            }
            
            if (!$product->canAddToCart(
                    $cartProductRow['quantity'],
                    $cartProductRow['product_variation_id'])) {
                
                $isValid = false;
                if ($quantity = $product->getQuantity(
                        $cartProductRow['product_variation_id'],
                        true)) {
                    
                    $this->updateItem($cartProductRow['id'], $quantity);
                    Ecart::message()->addError(
                        Ecart::translate('checkout')->__(
                            "Only %s item(s) of '%s' are available",
                            $quantity,
                            $cartProductRow['name']
                        )
                    );
                } else {
                    $this->deleteItem($cartProductRow['id']);
                    Ecart::message()->addError(
                        Ecart::translate('checkout')->__(
                            "Product '%s' is out of stock",
                            $cartProductRow['name']
                        )
                    );
                }
            }
        }
        if (!$isValid) {
            Ecart::message()->addNotice(
                Ecart::translate('checkout')->__(
                    'Due errors while checking your shopping cart contents, we made some changes to it. Check it out now'
                )
            );
        }
        return $isValid;
    }

    /**
     * Return rowset of shopping_cart_product table
     * 
     * @return Zend_Db_Table_Rowset_Abstract
     */
    public function getRows()
    {
        return Ecart::single('checkout/cart_product')
            ->findByCart($this->getCartId());
    }

    /**
     *
     * @return float
     */
    public function getTotalPrice()
    {
        return Ecart::single('checkout/cart_product')->fetchOne(
            'SUM(quantity*final_price)', 
            'shopping_cart_id = ' . $this->getCartId() 
        );
    }

    /**
     *
     * @return float
     */
    public function getTotalWeight()
    {
        return Ecart::single('checkout/cart_product')->fetchOne(
            'SUM(quantity*final_weight)', 
            'shopping_cart_id = ' . $this->getCartId() 
        );
    }
    
    /**
     * Return count shopping cart items
     *
     * @return int  
     */
    public function getCount()
    {
        $count = 0;
        $items = Ecart::single('checkout/cart_product')
            ->getProductsSimple($this->getCartId());
        foreach ($items as $item) {
            if ($item['decimal']) {
                $count++;
            } else { 
                $count += (int)$item['quantity'];
            }
        }
        return $count;
    }

    /**
     * Return customer id
     *
     * @return int
     */
    public function getCustomerId()
    {
        $customerId = $this->getCustomerIdById(
            $this->getCartId()
        );
        return $customerId ? $customerId : 0;
    }

    /**
     * Merge shopping carts
     *
     * @return bool
     */
    public function merge()
    {
        if (!$customerId = Ecart::getCustomerId()) {
            return false;
        }
        $previousCartRow = $this->fetchRow('customer_id = ' . $customerId);
        if ($previousCartRow && $previousCartRow->id != $this->getCartId()) {

            $collection = $this->getAdapter()->select()
                ->from(
                        array('ccp' => self::$_db_prefix . 'checkout_cart_product'),
                        array('*', 'checkout_cart_product_id' => 'id')
                )
                ->joinLeft(
                    array('ccpa' => self::$_db_prefix . 'checkout_cart_product_attribute'),
                    'ccp.id = ccpa.shopping_cart_product_id'
                )
                ->where('ccp.shopping_cart_id = ?', $previousCartRow->id)
                ;
            $previousCartProducts = $this->getAdapter()
                ->fetchAll($collection->__toString());
            $result = array();
            foreach ($previousCartProducts as $p) {
                if (!isset($result[$p['checkout_cart_product_id']])) {
                    $result[$p['checkout_cart_product_id']] = array(
                        'product_id' => $p['product_id'],
                        'quantity' => $p['quantity']
                    );
                }
                $productAttributeId = $p['product_attribute_id'];
                if (!empty($productAttributeId)) {
                    $result[$p['checkout_cart_product_id']]['attributes'][$productAttributeId] =
                        $p['product_attribute_value']
                    ;
                }
            }
            $removedShoppingProductIds = array();
            foreach ($result as $shopppingCartProductId => $product) {

                $attributes = isset($product['attributes']) ?
                    $product['attributes'] : array();
                $clon = $this->_getClon(
                    $product['product_id'], $attributes
                );
                if (!$clon) {
                    continue;
                }
                $this->updateItem(
                    $clon['id'], $clon['quantity'] + $product['quantity']
                );
                $removedShoppingProductIds[] = $shopppingCartProductId;

            }
            if (count($removedShoppingProductIds)) {
                Ecart::single('checkout/cart_product')->delete(
                    $this->getAdapter()->quoteInto('id IN (?)', $removedShoppingProductIds)
                );
            }
            Ecart::single('checkout/cart_product')->update(
                array('shopping_cart_id' => $this->getCartId()),
                $this->getAdapter()->quoteInto(
                    'shopping_cart_id = ?', $previousCartRow->id
                )
            );
            $this->delete(
                $this->getAdapter()->quoteInto('customer_id = ?', $customerId)
            );
        }
        $this->update(
            array('customer_id' => $customerId),
            $this->getAdapter()->quoteInto('id = ?', $this->getCartId())
        );
        return true;
    }
    
    /**
     * Remove all products from cart
     *
     * @return int
     */
    public function clear()
    {
        return Ecart::single('checkout/cart_product')->delete(
            'shopping_cart_id = ' . (int) $this->getId()
        );
    }
}