<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Install
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Install
 * @subpackage  Model
 * @author      Ecart Core Team <core@ecartcommerce.com>
 */
class Ecart_Install_Model_Installer
{
    /**
     * Execute multiqueries
     * 
     * @param string $sql
     * @return Ecart_Install_Model_Installer provides fluent interface
     */
    public function run($sql)
    {
        $tries = 0;
        $stmts = $this->_splitMultiQuery($sql);
        foreach ($stmts as $stmt) {
            do {
                $retry = false;
                try {
                    Ecart::db()->query($stmt);
                } catch (Exception $e) {
                    if ($e->getMessage() == 'SQLSTATE[HY000]: General error: 2013 Lost connection to MySQL server during query') {
                        $retry = true;
                        $tries++;
                    } else {
                        Ecart::message()->addError($e->getMessage());
                    }
                }
            } while ($retry && $tries < 10);
        }
        
        return $this;
    }
    
    /**
     * Split multi statement query
     *
     * @copyright  Copyright (c) 2008 Irubin Consulting Inc. DBA Varien (http://www.varien.com)
     * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
     * @param $sql string
     * @return array
     */
    private function _splitMultiQuery($sql)
    {
        $parts = preg_split(
            '#(;|\'|"|\\\\|//|--|\n|/\*|\*/)#',
            $sql,
            null,
            PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE
        );

        $q = false;
        $c = false;
        $stmts = array();
        $s = '';

        foreach ($parts as $i=>$part) {
            // strings
            if (($part==="'" || $part==='"') && ($i===0 || $parts[$i-1]!=='\\')) {
                if ($q===false) {
                    $q = $part;
                } elseif ($q===$part) {
                    $q = false;
                }
            }

            // single line comments
            if (($part==='//' || $part==='--') && ($i===0 || $parts[$i-1]==="\n")) {
                $c = $part;
            } elseif ($part==="\n" && ($c==='//' || $c==='--')) {
                $c = false;
            }

            // multi line comments
            if ($part==='/*' && $c===false) {
                $c = '/*';
            } elseif ($part==='*/' && $c==='/*') {
                $c = false;
            }

            // statements
            if ($part===';' && $q===false && $c===false) {
                if (trim($s)!=='') {
                    $stmts[] = trim($s);
                    $s = '';
                }
            } else {
                $s .= $part;
            }
        }
        if (trim($s)!=='') {
            $stmts[] = trim($s);
        }

        return $stmts;
    }
    
    /**
     * Retrieve table name with table prefix
     * 
     * @param string $table
     * @return string
     */
    public function getTable($table)
    {
        return Ecart::config()->db->prefix . $table;
    }
    
    /**
     * Add row to core_module table
     * 
     * @param string $module_code
     * @return void
     */
    public function registerModule($module)
    {
        $config = Ecart::single('core/module')->getConfig($module);
        
        $row = array(
            'package'   => $config[key($config)]['package'],
            'code'      => key($config),
            'name'      => $config[key($config)]['name'],
            'version'   => $config[key($config)]['version'],
            'is_active' => 1
        );
        
        Ecart::single('core/module')->insert($row);
    }
    
    /**
     * Updates module row in core_module table
     * 
     * @param string $module_code
     * @return unknown_type
     */
    public function updateModule($module)
    {
        if (!$module_row = Ecart::single('core/module')->fetchRow("code = '{$module}'")) {
            $module_row = Ecart::single('core/module')->createRow();
            $module_row->is_active = 1;
        }
        
        $config = Ecart::single('core/module')->getConfig($module);
        
        $module_row->setFromArray(array(
            'package'   => $config[key($config)]['package'],
            'code'      => key($config),
            'name'      => $config[key($config)]['name'],
            'version'   => $config[key($config)]['version']
        ));
        
        $module_row->save();
    }
    
    /**
     * Remove module row from core_module table 
     * 
     * @param string $module_code
     * @return void
     */
    public function unregisterModule($module)
    {
        Ecart::single('core/module')->delete(Ecart::db()->quoteInto('code = ?', $module));
    }
    
    /**
     * Disable foreign keys check while install is running
     * 
     * @return Ecart_Install_Model_Installer provides fluent interface
     */
    public function startSetup()
    {
        $this->run("
            SET SQL_MODE='';
            SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
            SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO';
        ");
        
        return $this;
    }
    
    /**
     * Revert changed mysql options to previous
     * 
     * @return Ecart_Install_Model_Installer provides fluent interface
     */
    public function endSetup()
    {
        $this->run("
            SET SQL_MODE=IFNULL(@OLD_SQL_MODE,'');
            SET FOREIGN_KEY_CHECKS=IFNULL(@OLD_FOREIGN_KEY_CHECKS,0);
        ");
        
        return $this;
    }
    
}