<?php
/**
 * Ecart
 * 
 * This file is part of Ecart.
 * 
 * Ecart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Ecart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Ecart.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @copyright   Copyright 2008-2009 E-Cart LLC
 * @license     GNU Public License V3.0
 */

/**
 * 
 * @category    Ecart
 * @package     Ecart_Checkout
 * @subpackage  Method
 * @author      Ecart Core Team <core@ecartcommerce.com>
 * @abstract
 */
abstract class Ecart_Method_Payment_Model_Abstract extends Ecart_Method_Abstract
{
    /**
     * Payment Method unique code
     */
    protected $_section = 'payment';
    protected $_file = __FILE__;
    
    /**
     * Checkout data storage
     *
     * @var Zend_Session_Namespace
     */
    private $_storage;
    
    public function __construct()
    {
        parent::__construct();
        try {
            $writer = new Zend_Log_Writer_Stream(
                Ecart::config()->system->path . 
                Ecart::config()->log->main->payment
            );
            $this->_logger = new Zend_Log($writer);
        } catch (Exception $e) {}
    }

    /**
     *
     */
    public function preProcess() {}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return bool
     */
    public function pending(Ecart_Sales_Model_Order_Row $order) { return true;}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return bool
     */
    public function processing(Ecart_Sales_Model_Order_Row $order) {return true;}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return bool
     */
    public function cancel(Ecart_Sales_Model_Order_Row $order) {return true;}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return bool
     */
    public function refund(Ecart_Sales_Model_Order_Row $order) {return true;}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return bool
     */
    public function failed(Ecart_Sales_Model_Order_Row $order) {return true;}

    /**
     *
     * @param Ecart_Sales_Model_Order_Row $order
     * @return mixed
     */
    public function postProcess(Ecart_Sales_Model_Order_Row $order) {}

    /**
     * Return checkout data storage
     *
     * @return Zend_Session_Namespace
     */
    public function getStorage()
    {
        if (null === $this->_storage) {
            $this->_storage = new Zend_Session_Namespace(
                $this->_code . '-Checkout'
            );
            $this->_storage->setExpirationSeconds(600);
            $this->_storage->setExpirationHops(3);
        }
        return $this->_storage;
    }

    /**
     * Saved custom attributes
     * @see (form.phtml, setPaymentAction)
     * @return bool
     * @param array $data
     */
    public function saveData($data) 
    {
        return $this->getStorage()->data = $data;
    }

    /**
     * Clear payment storage
     */
    public function clear() 
    {
        $this->getStorage()->unsetAll();    
        $this->_storage = null;
    }

    /**
     *
     * @param string $message
     */
    public function log($message)
    {
        Ecart_FirePhp::log($this->_code . ' ' . $message);
        //Ecart::message()->addError($message);
        $this->_logger->info($message);
    }

    /**
     *
     * @param array $request
     * @return bool
     */
    public function isAllowed($request)
    {
        if (isset($this->_config->minOrderTotal) && 
            !empty($this->_config->minOrderTotal) &&
            intval($request['price']) < intval($this->_config->minOrderTotal)) 
        {
            return false;
        }
        
        if (isset($this->_config->maxOrderTotal) && 
            !empty($this->_config->maxOrderTotal) &&
            intval($request['price']) > intval($this->_config->maxOrderTotal)) 
        {
            return false;
        }
        
        // get list of payments that is allowed by selected shipping method, and compare with requested payment method
        $shipping = Ecart::single('checkout/checkout')->shipping();
        $payments = $shipping->config()->payments->toArray();
        if (count($payments) && !in_array($this->getCode(), $payments)) {
            return false;
        }
        
        // get list of shippings that is allowed by this method and compare with selected shipping method
        $shippings = $this->_config->shippings->toArray();
        if (count($shippings) && !in_array($shipping->getCode(), $shippings)) {
            return false;
        }
        
        if (!isset($this->_config->geozone) 
            || !intval($this->_config->geozone)) {

            return true;
        }
        
        $zoneId = null;
        if (isset($request['zone_id'])) {
            $zoneId = $request['zone_id'];
        } elseif (isset($request['zone']['id'])) {
            $zoneId = $request['zone']['id'];
        }
        if (!isset($request['country']['id'])) {
            return false;
        }

        //Zend_Debug::dump($request);
        return Ecart::single('location/geozone_zone')->inGeozone(
            $this->_config->geozone,
            $request['country']['id'],
            $zoneId
        );
        
    }

    /**
     * @return string
     */
    public function getTitle()
    {
        $title = isset($this->_config->title) ? 
            $this->_config->title : $this->_title;
        
        return $this->getTranslator()->__($title);
    }
    
    /**
     * Convert abstract total of order 
     * to payment baseCurrency
     * 
     * @param float $amount
     * @return float
     */
    public function getAmountInBaseCurrency($amount)
    {
        return Ecart::single('locale/currency')
            ->to($amount, $this->getBaseCurrencyCode());
    }
    
    /**
     * Retrieve baseCurrency code
     * Used for online transactions
     * 
     * @return string 
     */
    public function getBaseCurrencyCode()
    {
        $baseCurrency = $this->_config->currency;
        if (null === $baseCurrency) {
            $baseCurrency = Ecart::config()->main->store->baseCurrency;
        }
        return (null === $baseCurrency || strlen($baseCurrency) != 3) ?
            'USD' : $baseCurrency;
    }
    
    /**
     * Only Ecart payments are supported
     * @todo return module with category name
     *  example: Ecart_PaymentPaypal
     *  
     * @return Ecart_Translator
     */
    public function getTranslator()
    {
        return Ecart::translate('Payment' . current(explode('_', $this->_code)));
    }
}